;    Z80 emulator with CP/M support. The Z80-specific instructions themselves actually aren't
;    implemented yet, making this more of an i8080 emulator.
;    
;    Copyright (C) 2010 Sprite_tm
;
;    This file is part of avrcpm.
;
;    avrcpm is free software: you can redistribute it and/or modify it
;    under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    avrcpm is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with avrcpm.  If not, see <http://www.gnu.org/licenses/>.
;
;    $Id$
;


#ifndef F_CPU
	#define F_CPU  20000000        /* system clock in Hz; defaults to 20MHz */
#endif
#ifndef BAUD
	#define BAUD   38400           /* console baud rate */
#endif
 
#define PARTID 0x52		/* Partition table id */
				/* http://www.win.tue.nl/~aeb/partitions/partition_types-1.html */
#define RAMDISKNR 'I'-'A'	/* Driveletter for first RAM disk */
#define RAMDISKCNT 1		/* Number of RAM disks */

#define UBRR_VAL  ((F_CPU+BAUD*8)/(BAUD*16)-1)  /* clever rounding */

#define	RXBUFSIZE 64		/* USART recieve buffer size. Must be power of 2 */
#define	TXBUFSIZE 64		/* USART transmit buffer size. Must be power of 2 */

#define IPLADDR	0x2000		/* Bootloader load address */

#define DRAM_WAITSTATES 1	/* Number of additional clock cycles for dram read access */
#define REFR_RATE   64000       /* dram refresh rate in cycles/s. */
				/* Most drams need 1/15.6µs. */
#define REFR_PRE    8           /* timer prescale factor */
#define REFR_CS     0x02        /* timer clock select for 1/8  */
#define REFR_CNT    F_CPU / REFR_RATE / REFR_PRE




#define EM_Z80	0		/* we don't have any z80 instructions yet */

.equ BOOT_DEBUG  = 0
.equ MMC_DEBUG   = 0
.equ INS_DEBUG   = 0
.equ MEMTEST     = 1
.equ BOOTWAIT    = 1
.equ PORT_DEBUG  = 0
.equ DISK_DEBUG  = 0		/* Increase for more debugging */
.equ HOSTRW_DEBUG= 0
.equ MEMFILL_CB  = 1
.equ STACK_DBG   = 0
.equ PRINT_PC    = 0

;Port declarations

; Port D
.equ rxd    = 0
.equ txd    = 1
.equ ram_oe = 2
.equ ram_a8 = 3
.equ mmc_cs = 4
.equ ram_a5 = 5
.equ ram_a6 = 6
.equ ram_a7 = 7

.equ P_OE  = PORTD
.equ P_AH  = PORTD
.equ P_A8  = PORTD
.equ P_MMC_CS = PORTD
	             ; ram_a[7..5]
.equ RAM_AH_MASK = (1<<ram_a8)|(1<<ram_a7)|(1<<ram_a6)|(1<<ram_a5)
.equ PD_OUTPUT_MASK = (1<<mmc_cs) | (1<<ram_oe) | RAM_AH_MASK


;Port B
.equ ram_a4 =	0
.equ ram_a3 =	1
.equ ram_a2 =	2
.equ ram_a1 =	3
.equ mmc_mosi =	3
.equ ram_a0 =	4
.equ mmc_miso =	4
.equ ram_ras =	5
.equ mmc_sck =	5


.equ P_RAS = PORTB
.equ P_AL  = PORTB
			; ram_a[4..0]
.equ RAM_AL_MASK = (1<<ram_a4)|(1<<ram_a3)|(1<<ram_a2)|(1<<ram_a1)|(1<<ram_a0)
.equ PB_OUTPUT_MASK = (1<<ram_ras) | RAM_AL_MASK

;Port C
.equ ram_d0 =	0
.equ ram_d1 =	1
.equ ram_d2 =	2
.equ ram_d3 =	3
.equ ram_w  =	4
.equ ram_cas=	5

.equ P_DQ  = PORTC
.equ P_W   = PORTC
.equ P_CAS = PORTC
.equ RAM_DQ_MASK = (1<<ram_d3)|(1<<ram_d2)|(1<<ram_d1)|(1<<ram_d0)
.equ PC_OUTPUT_MASK = (1<<ram_cas)|(1<<ram_w)


;Flag bits in z_flags
.equ ZFL_S	=	7
.equ ZFL_Z	=	6
.equ ZFL_H	=	4
.equ ZFL_P	=	2
.equ ZFL_N	=	1
.equ ZFL_C	=	0

;Register definitions

.def	_tmp	= r0	;  0
.def	_0	= r1

.def	_RAS0	= r2
.def	_CAS0	= r3
.def	_OE	= r4
.def	_WE	= r5
.def	_255	= r6

.def	z_a	= r9

;.def	stx_bitcount = r9
;.def	stx_dr	= r10
.def	srx_lastedgel = r10
.def	srx_lastedgeh = r11

.def	insdecl	= r12	;
.def	insdech	= r13	;
.def	z_spl	= r14
.def	z_sph	= r15	;
.def	temp	= r16 	;
.def	temp2	= r17 	;
.def	temp3	= r18
.def	temp4	= r19
.def	z_flags	= r20	;
			;
.def	opl	= r22	;
.def	oph	= r23	;
.def	z_pcl	= r24	;
.def	z_pch	= r25	;
; xl 		;r26
; xh		;r27
; yl		;r28
; yh		;r29
; zl		;r30	;
; zh		;r31	;



#if defined __ATmega8__
.equ	flags	= TWBR
.equ	P_PUD	= SFIOR
#else
.equ	flags	= GPIOR0
.equ	P_PUD	= MCUCR
#endif

; Flags:
	.equ	hostact	= 7		;host active flag
	.equ	hostwrt	= 6		;host written flag
	.equ	rsflag	= 5		;read sector flag
	.equ	readop	= 4		;1 if read operation
	.equ	trace	= 0

; This is the base z80 port address for clock access
#define	TIMERPORT 0x40
#define TIMER_CTL   TIMERPORT
#define TIMER_MSECS TIMERPORT+1
#define TIMER_SECS  TIMER_MSECS+2

#define starttimercmd	1
#define quitTimerCmd	2
#define printTimerCmd	15
#define uptimeCmd	16

#if defined __ATmega8__
.equ	RXTXDR0	= UDR
.equ	UCSR0A	= UCSRA
.equ	 UDRE0	= UDRE
.equ	UCSR0B	= UCSRB
.equ	 RXCIE0	= RXCIE
.equ	 UDRIE0	= UDRIE
.equ	 RXEN0	= RXEN
.equ	 TXEN0	= TXEN
.equ	UCSR0C	= UCSRC
.equ	 UCSZ00	= UCSZ0
.equ	 UCSZ01	= UCSZ1
.equ	UBRR0H	= UBRRH
.equ	UBRR0L	= UBRRL
.equ	OCR2A	= OCR2
.equ	OC2Aaddr= OC2addr
.equ	TCCR2A	= TCCR2
.equ	TCCR2B	= TCCR2
.equ	TIMSK1	= TIMSK
.equ	TIMSK2	= TIMSK
.equ	OCIE2A	= OCIE2
.equ	TIFR1	= TIFR
.equ	ICIE1	= TICIE1
#else
.equ	RXTXDR0	= UDR0
#endif

; vim:set ts=8 noet nowrap


